package com.icing.dialer.modem

import com.theeasiestway.opus.Opus
import java.nio.ByteBuffer
import java.nio.ShortBuffer

class AudioCodec {
    private var encoder: Long = 0
    private var decoder: Long = 0
    private val opus = Opus()
    
    init {
        // Initialize Opus encoder and decoder
        encoder = opus.encoderCreate(
            FSKConstants.SAMPLE_RATE,
            1, // Mono
            Opus.OPUS_APPLICATION_VOIP
        )
        
        decoder = opus.decoderCreate(
            FSKConstants.SAMPLE_RATE,
            1 // Mono
        )
        
        // Configure encoder
        opus.encoderSetBitrate(encoder, FSKConstants.OPUS_BITRATE)
        opus.encoderSetComplexity(encoder, FSKConstants.OPUS_COMPLEXITY)
        opus.encoderSetSignal(encoder, Opus.OPUS_SIGNAL_VOICE)
        opus.encoderSetPacketLossPerc(encoder, 10) // Expect 10% packet loss
        opus.encoderSetInbandFEC(encoder, 1) // Enable FEC
        opus.encoderSetDTX(encoder, 1) // Enable discontinuous transmission
    }
    
    fun encode(audioData: ShortArray): ByteArray {
        val maxEncodedSize = 1024
        val encodedData = ByteArray(maxEncodedSize)
        
        val encodedLength = opus.encode(
            encoder,
            audioData,
            FSKConstants.OPUS_FRAME_SIZE,
            encodedData
        )
        
        return if (encodedLength > 0) {
            encodedData.copyOf(encodedLength)
        } else {
            throw RuntimeException("Opus encoding failed with error: $encodedLength")
        }
    }
    
    fun decode(encodedData: ByteArray): ShortArray {
        val decodedData = ShortArray(FSKConstants.OPUS_FRAME_SIZE)
        
        val decodedSamples = opus.decode(
            decoder,
            encodedData,
            decodedData,
            FSKConstants.OPUS_FRAME_SIZE,
            0 // No packet loss
        )
        
        return if (decodedSamples > 0) {
            decodedData.copyOf(decodedSamples)
        } else {
            throw RuntimeException("Opus decoding failed with error: $decodedSamples")
        }
    }
    
    fun decodeLost(): ShortArray {
        val decodedData = ShortArray(FSKConstants.OPUS_FRAME_SIZE)
        
        val decodedSamples = opus.decode(
            decoder,
            null,
            decodedData,
            FSKConstants.OPUS_FRAME_SIZE,
            1 // Packet lost
        )
        
        return if (decodedSamples > 0) {
            decodedData.copyOf(decodedSamples)
        } else {
            ShortArray(FSKConstants.OPUS_FRAME_SIZE) // Return silence
        }
    }
    
    fun release() {
        if (encoder != 0L) {
            opus.encoderDestroy(encoder)
            encoder = 0
        }
        if (decoder != 0L) {
            opus.decoderDestroy(decoder)
            decoder = 0
        }
    }
    
    protected fun finalize() {
        release()
    }
}