# DryBox - Secure Voice Communication System

A PyQt5-based application demonstrating secure voice communication using the Noise XK protocol, Codec2 audio compression, and 4FSK modulation.

## Features

- **Secure Communication**: End-to-end encryption using Noise XK protocol
- **Audio Compression**: Codec2 (3200bps) for efficient voice transmission
- **Modulation**: 4FSK (4-level Frequency Shift Keying) for robust transmission
- **GSM Network Simulation**: Simulates realistic GSM network conditions
- **Real-time Audio**: Playback and recording capabilities
- **Visual Feedback**: Waveform displays and signal strength indicators

## Requirements

- Python 3.7+
- PyQt5
- NumPy
- pycodec2
- Additional dependencies in `requirements.txt`

## Installation

1. Install system dependencies:
   ```bash
   ./install_audio_deps.sh
   ```

2. Install Python dependencies:
   ```bash
   pip install -r requirements.txt
   ```

## Running the Application

Simply run:
```bash
python3 UI/main.py
```

The application will automatically:
- Start the GSM network simulator
- Initialize two phone clients
- Display the main UI with GSM status panel

## Usage

### Phone Controls
- **Click "Call" button** or press `1`/`2` to initiate/answer calls
- **Ctrl+1/2**: Toggle audio playback for each phone
- **Alt+1/2**: Toggle audio recording for each phone

### GSM Settings
- **Click "Settings" button** or press `Ctrl+G` to open GSM settings dialog
- Adjust signal strength, quality, noise, and network parameters
- Use presets for quick configuration (Excellent/Good/Fair/Poor)

### Other Controls
- **Space**: Run automatic test sequence
- **Ctrl+L**: Clear debug console
- **Ctrl+A**: Audio processing options menu

## Architecture

- **main.py**: Main UI application
- **phone_manager.py**: Manages phone instances and audio
- **protocol_phone_client.py**: Implements the secure protocol stack
- **noise_wrapper.py**: Noise XK protocol implementation
- **gsm_simulator.py**: Network simulation relay
- **gsm_status_widget.py**: Real-time GSM status display

## Testing

The automatic test feature (`Space` key) runs through a complete call sequence:
1. Initial state verification
2. Call initiation
3. Call answering
4. Noise XK handshake
5. Voice session establishment
6. Audio transmission
7. Call termination